/* grid.c - Grid handling code.
 *
 * Copyright (C) 1999 Havoc Pennington
 * Copyright (C) 2000 Iain Holmes  <ih@csd.abdn.ac.uk>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "app.h"
#include "grid.h"
#include "edit.h"
#include <config.h>
#include <gnome.h>

#include <gnome-canvas-grid.h>
#include <ie-canvas-layer.h>

gint  item_event(GnomeCanvasItem* item, GdkEvent* event, gpointer data);
void grid_set_alpha(Grid* g, guint x, guint y, 
		    guchar a);
void grid_fill_recursive(Grid* g, guint x, guint y,
			 gboolean *already,
			 guchar start_r, guchar start_g, guchar start_b,
			 guchar start_a, GdkColor* color, guchar a);
void grid_fill(Grid* g, guint x, guint y, 
	       GdkColor* color);
gint rect_cb (GnomeCanvasItem *item,
	      GdkEvent *event, 
	      gpointer data);

#define SQUARE_SIZE 17.0
#define OUTLINE_WIDTH 3.0

Grid* 
grid_new(GtkWidget* app, GtkWidget* canvas, GtkWidget* preview, 
         GtkWidget* status, GtkWidget* picker, GtkAdjustment* alpha, 
         GtkWidget *mini, guint width, guint height)
{
  Grid* g;

  g = g_new0(Grid, 1);

  g->tool = IET_FG;
  g->mode = IEM_NORMAL;
  g->dirty = FALSE;
  g->app = app;
  g->canvas = canvas;
  g->preview = preview;
  g->mini = mini;
  g->status = status;
  g->picker = picker;
  g->alpha = alpha;

  gtk_drawing_area_size(GTK_DRAWING_AREA(g->preview), width+5, height+5);
  gtk_widget_queue_resize(g->preview); /* workaround bug in some 1.2.x */

  g->width = width;
  
  g->height = height;

  g->pixbuf = gdk_pixbuf_new (ART_PIX_RGB, TRUE, 8, width, height);
  /* Clear the pixbuf. */
  memset (gdk_pixbuf_get_pixels (g->pixbuf), 0x00, width * height * 4);

  /* align to 4-byte boundary, and multiply by 3 for the RGB triplet */
  g->rowstride = gdk_pixbuf_get_rowstride (g->pixbuf); 

  g_assert(g->rowstride % 4 == 0);

  g->current_color.red = 0;
  g->current_color.green = 0;
  g->current_color.blue = 0;
  g->current_color.pixel = gdk_rgb_xpixel_from_rgb(0x000000);

  g->black = g->current_color;

  gnome_canvas_set_scroll_region(GNOME_CANVAS(g->canvas),
				 0.0, 0.0,
				 10.0 * g->width,
				 10.0 * g->height);
  
  /* In order: bottom->top */
  g->cboard = gnome_canvas_item_new (GNOME_CANVAS_GROUP (GNOME_CANVAS(g->canvas)->root),
				     ie_canvas_chequer_board_get_type (),
				     "x", 0.0,
				     "y", 0.0,
				     "width", (gdouble)width * 10.0,
				     "height", (gdouble)height * 10.0,
				     "square_size", 16.0,
				     NULL);
  g->mainlayer = gnome_canvas_item_new (GNOME_CANVAS_GROUP (GNOME_CANVAS(g->canvas)->root),
					ie_canvas_layer_get_type (),
					"x", 0.0,
					"y", 0.0,
					"width", (double) width,
					"height", (double) height,
					"square_size", 10.0,
					NULL);
  g->gridlines = gnome_canvas_item_new (GNOME_CANVAS_GROUP (GNOME_CANVAS(g->canvas)->root),
					ie_canvas_grid_get_type (),
					"x", 0.0,
					"y", 0.0,
					"width", (double) width,
					"height", (double) height,
					"thickness", 2.0,
					"square_size", 10.0,
					"fill_color_rgba", 0x000000FF,
					NULL);
  /* Copy the pixbuf in. */
  grid_set_from_pixbuf (g, g->pixbuf);

  /* Set the zoom level */
  gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (g->canvas), 1.0);
  gtk_signal_connect (GTK_OBJECT (g->mainlayer), "event",
		      GTK_SIGNAL_FUNC (item_event), g);
  g->sel = NULL;
  return g;
}

void
grid_destroy(Grid* g)
{
  gdk_pixbuf_unref (g->pixbuf);
  g_free(g);
}

const guchar*
grid_pixel(Grid* g, guint x, guint y)
{
  guchar *pixels;
  guint i = y*g->rowstride + x*4;

  pixels = gdk_pixbuf_get_pixels (g->pixbuf);
  return pixels + i;
}

const guchar* 
grid_alpha(Grid* g, guint x, guint y)
{
  guchar *pixels;
  guint i = y*g->rowstride + x*4;

  pixels = gdk_pixbuf_get_pixels (g->pixbuf);
  return pixels + i + 3; /* +3 because alpha is the 4th char in a group and i
			    points to the beginning of the group */
}

void 
grid_update_item(Grid* g)
{
  gnome_canvas_item_request_update (g->mainlayer);
/*   gtk_widget_queue_draw(g->preview); */
}

void 
grid_set(Grid* g, guint x, guint y, 
         guchar r, guchar gr, guchar b, guchar a,
	 gboolean update)
{
  guchar* pixels = gdk_pixbuf_get_pixels (g->pixbuf);
  guchar* pixel = pixels + y*g->rowstride + x*4;

  mode_convert_pixel (g, x, y, &r, &gr, &b, &a);
  *pixel = r;
  ++pixel;
  *pixel = gr;
  ++pixel;
  *pixel = b;
  ++pixel;
  *pixel = a;
  
  if (update)
    ie_canvas_layer_update_cell (g->mainlayer, (double)x, (double)y);
}


void
grid_set_gdk(Grid* g, guint x, guint y,
             GdkColor* color, guchar a, gboolean update)
{
    grid_set(g, x, y, color->red/256, 
	     color->green/256, color->blue/256, a, update);
}

void 
grid_set_alpha(Grid* g, guint x, guint y, 
	       guchar a)
{
  guchar *pixels = gdk_pixbuf_get_pixels (g->pixbuf);
  guchar* alpha = pixels + (y*g->rowstride + (x * 4)) + 3;
  
  *alpha = a;
  
  grid_update_item(g);
}

void
grid_set_from_pixbuf(Grid* g, GdkPixbuf *pb)
{
  GtkArg args[1];
  guint x, y;
  gint w, h, rowstride;
  gboolean has_alpha;
  guchar *rgb_data, *rgb;
  guchar *buf, *bufptr, *old = NULL;
  GdkPixbuf *new_pb;

  w = gdk_pixbuf_get_width (pb);
  h = gdk_pixbuf_get_height (pb);
  
  g_return_if_fail(w == g->width);
  g_return_if_fail(h == g->height);

  buf = bufptr = g_new (guchar, w * h * 4);
  has_alpha = gdk_pixbuf_get_has_alpha (pb);
  rgb = gdk_pixbuf_get_pixels (pb);
  rowstride = gdk_pixbuf_get_rowstride (pb);

  if (has_alpha)
    memcpy (buf, rgb, w * h * 4); /* Just do a fast direct memcpy */
  else
    {
      for (y = 0; y < h; y++)
	for (x = 0; x < w; x++)
	  {
	    rgb_data = rgb + (y * rowstride) + (x * 3);
	    bufptr = buf + (y * w * 4) + (x * 4);
	    memcpy (bufptr, rgb_data, 3); /* Copy the RGB from the image */
	    bufptr[3] = 0xFF; /* Add the A */
	  }
    }

  new_pb = gdk_pixbuf_new_from_data (buf, ART_PIX_RGB, TRUE,
				     w, h, w * 4,
				     NULL, NULL);

  /* Get the old grid pixbuf */
  args[0].name = "IECanvasLayer::pixbuf";
  gtk_object_getv (GTK_OBJECT (g->mainlayer),
		   1,
		   args);

  old = GTK_VALUE_POINTER (args[0]);
  gdk_pixbuf_unref (g->pixbuf);
  g->pixbuf = new_pb;
  gdk_pixbuf_ref (g->pixbuf);
  gnome_canvas_item_set (g->mainlayer,
			 "pixbuf", gdk_pixbuf_get_pixels (new_pb),
			 "rowstride", gdk_pixbuf_get_rowstride (new_pb),
			 NULL);

  /* Free the old buffer */
  if (old)
    g_free (old);
}

GdkPixbuf*
grid_to_pixbuf (Grid *grid)
{
  gdk_pixbuf_ref (grid->pixbuf);
  return grid->pixbuf;
}

void
grid_fill_recursive(Grid* g, guint x, guint y,
		    gboolean* already,
                    guchar start_r, guchar start_g, guchar start_b,
                    guchar start_a, GdkColor* color, guchar a)
{
  guchar *pixels;
  guchar* pixel;
  gboolean *revisit = already + (y * g->height + x);

  if (*revisit)
    return;

  *revisit = TRUE;

  pixels = gdk_pixbuf_get_pixels (g->pixbuf);
  pixel = pixels + y*g->rowstride + x*4;

  if (pixel[0] == start_r && pixel[1] == start_g && pixel[2] == start_b &&
      pixel[3] == start_a)
    {
      if (color)
        {
          grid_set_gdk(g, x, y, color, a, FALSE);
        }
      else
        {
          grid_set_gdk(g, x, y, &g->black, 0, FALSE);
        }

      if (x < (g->width - 1))
	grid_fill_recursive(g, x+1, y, already, start_r, start_g, start_b, 
			    start_a, color, a);
      
      if (x > 0)
	grid_fill_recursive(g, x-1, y, already, start_r, start_g, start_b, 
			    start_a, color, a);

      if (y < (g->height - 1))
	grid_fill_recursive(g, x, y+1, already, start_r, start_g, start_b, 
			    start_a, color, a);

      if (y > 0)
	grid_fill_recursive(g, x, y-1, already, start_r, start_g, start_b, 
			    start_a, color, a);
    }
  else
    {
      return;
    }
}

void 
grid_fill(Grid* g, guint x, guint y, 
          GdkColor* color) /* if color is NULL, fill transparent */
{
  /* This isn't going to be efficient, but it's easy. It's a small
     icon we're editing. */
  guchar* pixels = gdk_pixbuf_get_pixels (g->pixbuf);
  guchar* pixel = pixels + y*g->rowstride + x*4;

  guchar start_r = pixel[0]; 
  guchar start_g = pixel[1];
  guchar start_b = pixel[2];
  guchar start_a = pixel[3];
  
  guchar alpha = (guchar)g->alpha->value;
  gboolean *already = g_new0 (gboolean, g->width * g->height);

  grid_fill_recursive(g, x, y, already, start_r, start_g, start_b, 
                      start_a, color, alpha);
  g_free (already);
}

/* Item event handlers */

gint
rect_cb (GnomeCanvasItem *item,
	 GdkEvent *event, 
	 gpointer data)
{
  int result;
  Grid *grid;

  grid = (Grid *) data;
  g_return_if_fail (grid != NULL);

  /* Pass all events upwards */
  gtk_signal_emit_by_name (GTK_OBJECT (grid->mainlayer), "event",
			   event, &result);
  
  return result;
}

gint  
item_event(GnomeCanvasItem* item, GdkEvent* event, gpointer data)
{
  Grid *grid = data;
  IECanvasLayer *gi = IE_CANVAS_LAYER (item);
  IESelection *selection;
  static gdouble ox = 0.0, oy = 0.0;
  gint r_x, r_y;
  static gint s_x = 0, s_y = 0;
  double size;
  GtkArg arg[1];

  /* Get the square size */
  arg[0].name = "IECanvasLayer::square_size";
  gtk_object_getv (GTK_OBJECT (gi),
		   1, arg);
  g_assert (arg[0].type == GTK_TYPE_DOUBLE);
  size = GTK_VALUE_DOUBLE (arg[0]);
  
  switch (event->type)
    {
    case GDK_BUTTON_PRESS:
      {
	r_x = (gint)(event->button.x / size);
	r_y = (gint)(event->button.y / size);
        switch (grid->tool)
          {
	  case IET_FG:
            switch (event->button.button)
              {
              case 1:
                grid_set_gdk(grid, r_x, r_y, &grid->current_color, 
			     (gint)grid->alpha->value, TRUE);
		ie_app_dirty (grid->app);
                return TRUE;
                break;
              case 2:
                /* makes flood fill work right - haha. */
                grid_set_gdk(grid, r_x, r_y, &grid->black, 0x0, TRUE);
		ie_app_dirty (grid->app);
		return TRUE;
                break;
              }
	    break;

	  case IET_FILL:
            switch (event->button.button)
              {
              case 1:
                grid_fill (grid, r_x, r_y, &grid->current_color);
		ie_update_grid (grid->app);
                break;
              case 2:
                /* fill transparent */
                grid_fill (grid, r_x, r_y, NULL);
		ie_update_grid (grid->app);
                break;
              }
	    ie_app_dirty (grid->app);
	    return TRUE;
	    break;

	  case IET_EYEDROP:
            if (event->button.button == 1)
              {
                const guchar* pixel = grid_pixel(grid, r_x, r_y);

                grid->current_color.red = pixel[0]*256;
                grid->current_color.green = pixel[1]*256;
                grid->current_color.blue = pixel[2]*256;
                grid->current_color.pixel = 
                  gdk_rgb_xpixel_from_rgb((((guint)pixel[0]) << 16) +
                                          (((guint)pixel[1]) << 8) +
                                          pixel[2]);

                gnome_color_picker_set_i8(GNOME_COLOR_PICKER(grid->picker),
                                          pixel[0],
                                          pixel[1],
                                          pixel[2],
                                          255);
		grid->alpha->value = (gfloat)pixel[3];
		gtk_adjustment_value_changed (grid->alpha);
              }
	    break;
	  case IET_POINT:
	    {
	      GdkCursor *crosshairs;
	      selection = ie_selection_get (grid->app);
	      ox = r_x * size;
	      oy = r_y * size;
	      selection->x = r_x;
	      selection->y = r_y;
	      if (selection->selitem)
		gtk_object_destroy (GTK_OBJECT (selection->selitem));
	      selection->selitem = NULL;
	      crosshairs = gdk_cursor_new (GDK_CROSS);
	      gdk_window_set_cursor (grid->canvas->window, crosshairs);
	      return TRUE;
	    }
	    break;

	  case IET_BRUSH:
	    edit_paste_at (grid, r_x, r_y);
	    break;
	    
	  default:
	    return FALSE;
	  }
	break;
      }

    case GDK_BUTTON_RELEASE:
      r_x = (gint)(event->button.x / size);
      r_y = (gint)(event->button.y / size);
      if (event->button.button > 3 && event->button.state & GDK_CONTROL_MASK)
	{
	  switch (event->button.button)
	    {
	    case 4:
	      g_print ("Zoom in!\n");
	      break;
	    case 5:
	      g_print ("Zoom out!\n");
	      break;
	      default:
		break;
	    }
	  return FALSE;
	}
      
      if (grid->tool == IET_POINT)
	{
	  gint t_x, t_y;
	  selection = ie_selection_get (grid->app);

	  gdk_window_set_cursor (grid->canvas->window, NULL);
	  
	  if (r_x < selection->x)
	    {
	      t_x = selection->x;
	      selection->x = r_x;
	    }
	  else
	    t_x = r_x;
	  
	  if (r_y < selection->y)
	    {
	      t_y = selection->y;
	      selection->y = r_y;
	    }
	  else
	    t_y = r_y;
	  
	  selection->grid = grid;
	  selection->width = (t_x - selection->x) + 1;
	  selection->height = (t_y - selection->y) + 1;
	  
	  return FALSE;
	}
      break;
      
    case GDK_MOTION_NOTIFY:
      r_x = (gint)(event->motion.x / size);
      r_y = (gint)(event->motion.y / size);
      
      if (r_x < 0 || r_y < 0 ||
	  r_x > (gi->width - 1) || r_y > (gi->height - 1))
	return TRUE;
      
      if ((grid->tool == IET_POINT) &&
	  ((event->motion.state & GDK_BUTTON1_MASK) == GDK_BUTTON1_MASK))
	{
	  GnomeCanvasGroup *group;
	  gdouble x, y;
	  
	  x = MIN ((gint)size * grid->width, event->motion.x);
	  y = MIN ((gint)size * grid->height, event->motion.y);
	  
	  group = GNOME_CANVAS_GROUP (GNOME_CANVAS (grid->canvas)->root);
	  selection = ie_selection_get (grid->app);
	  if (selection->selitem == NULL)
	    {
	      selection->selitem = gnome_canvas_item_new (group,
							  gnome_canvas_rect_get_type (),
							  "x1", (double) ox,
							  "y1", (double) oy,
							  "x2", (double) r_x * size + size,
							  "y2", (double) r_y * size + size,
							  "fill_color_rgba", 0x8888FF55,
							  "outline_color", "white",
							  "width_pixels", 1,
							  NULL);
	      gtk_signal_connect (GTK_OBJECT (selection->selitem), "event",
				  GTK_SIGNAL_FUNC (rect_cb), grid);
	    }
	  else
	    {
	      double t_x1, t_y1, t_x2, t_y2;
	      if ((r_x * size) < ox)
		{
		  t_x1 = r_x * size;
		  t_x2 = ox;
		}
	      else
		{
		  t_x1 = ox;
		  t_x2 = r_x * size;
		}
	      
	      if ((r_y * size) < oy)
		{
		  t_y1 = r_y * size;
		  t_y2 = oy;
		}
	      else
		{
		  t_y1 = oy;
		  t_y2 = r_y * size;
		}
	      
	      gnome_canvas_item_set (selection->selitem,
				     "x1", t_x1,
				     "y1", t_y1,
				     "x2", t_x2 + size,
				     "y2", t_y2 + size,
				     NULL);
	    }
	  return TRUE;
	}
      else if (grid->tool == IET_FG && event->button.state & GDK_BUTTON1_MASK)
	{
	  grid_set_gdk (grid, r_x, r_y, &grid->current_color,
			(gint)grid->alpha->value, TRUE);
	  ie_app_dirty (grid->app);
	}
      else if (grid->tool == IET_FG && event->button.state & GDK_BUTTON2_MASK)
	{
	  grid_set_gdk (grid, r_x, r_y, &grid->black, 0x00, TRUE);
	  ie_app_dirty (grid->app);
	}
      
      if ((s_x == r_x) && (s_y == r_y) && (grid->sel != NULL))
	return TRUE;
      
      /* Display the co-ordinates and stuff in the statusbar */
      ie_update_statusbar (grid, r_x, r_y);
      
      if (grid->sel == NULL)
	{
	  grid->sel = gnome_canvas_item_new (gnome_canvas_root (item->canvas),
					     gnome_canvas_rect_get_type (),
					     "x1", (double) r_x * size,
					     "y1", (double) r_y * size,
					     "x2", (double) r_x * size + size, 
					     "y2", (double) r_y * size + size,
					     "fill_color", NULL,
					     "outline_color_rgba", 0xFFFFFFFF,
/*  					     "fill_color_rgba", 0xFF990055, */
					     "width_pixels", 2,
					     NULL);

	  gtk_signal_connect (GTK_OBJECT (grid->sel), "event",
			      GTK_SIGNAL_FUNC (rect_cb), grid);
	}
      else
	{
	  gnome_canvas_item_set (grid->sel,
				 "x1", (double) r_x * size,
				 "y1", (double) r_y * size,
				 "x2", (double) r_x * size + size,
				 "y2", (double) r_y * size + size,
				 NULL);
	}
      s_x = r_x;
      s_y = r_y;
      return TRUE;
      break;
      
    default:
      break;
    }
  
  return FALSE;
}
